package com.rjconsultores.ventaboletos.relatorios.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.ConferenciaFormularioFisico;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

public class RelatorioConferenciaFormularioFisico extends Relatorio {

	public RelatorioConferenciaFormularioFisico(Map<String, Object> parametros, Connection conexao) {
		super(parametros, conexao);

	}

	public void setLsDadosRelatorio(List<ConferenciaFormularioFisico> formularios) {
		this.setCollectionDataSource(new JRBeanCollectionDataSource(formularios));
	}

	private String getSql(Integer puntoventaId, Integer tipoventaId) {

		StringBuffer sql = new StringBuffer();
		sql.append("select distinct t1.*, t2.numfoliosistema as minlogico, t3.numfoliosistema as maxlogico ");
		sql.append("from  ");
		sql.append("(select  ");
		sql.append("  e.numcaja as ter,  ");
		sql.append("  p.nombpuntoventa as caixa,  ");
		sql.append("  c.numseriepreimpresa as serie,   ");
		sql.append("  u.cveusuario || ' - ' || u.nombusuario as usuario,  ");
		sql.append("  t.turno_id as turno,  ");
		sql.append("  min(c.fechorventa) as abertura,  ");
		sql.append("  max(c.fechorventa) as fechamento,  ");
		sql.append("  min(c.numfoliopreimpreso) as minfisico,  ");
		sql.append("  max(c.numfoliopreimpreso) as maxfisico,  ");
		sql.append("  (max(c.numfoliopreimpreso) + 1) - min(c.numfoliopreimpreso) as dif1, ");
		sql.append("  count(case when (c.indreimpresion = 1 and c.indstatusboleto = 'E') then -1 else 1 end) as total ");
		sql.append("from   ");
		sql.append("  caja c  ");
		sql.append("  inner join estacion e on e.estacion_id = c.estacion_id  ");
		sql.append("  inner join punto_venta p on p.puntoventa_id = c.puntoventa_id  ");
		sql.append("    and p.puntoventa_id = e.puntoventa_id  ");
		sql.append("  inner join usuario u on u.usuario_id = c.usuario_id  ");
		sql.append("  inner join turno t on t.turno_id = c.turno_id  ");
		sql.append("	where ");
		sql.append("  		c.empresacorrida_id = ? ");
		sql.append("  		and c.feccorte between ? and ? ");
		if (puntoventaId != null && puntoventaId != -1) {
			sql.append("  	and c.puntoventa_id = " + puntoventaId);
		}
		sql.append("  and c.activo <> 0 ");
		sql.append("  and c.motivocancelacion_id is null ");
		sql.append("  and c.numfoliopreimpreso is not null ");
		sql.append("  and c.numfoliosistema is not null ");
		sql.append("group by   ");
		sql.append("  e.numcaja,   ");
		sql.append("  p.nombpuntoventa,   ");
		sql.append("  c.numseriepreimpresa,   ");
		sql.append("  u.cveusuario || ' - ' || u.nombusuario,   ");
		sql.append("  t.turno_id  ");
		sql.append("having min(c.numfoliopreimpreso) > 0 ");
		sql.append("order by   ");
		sql.append("  u.nombusuario, min(c.fechorventa)) t1 ");
		sql.append("inner join (select numfoliosistema, numfoliopreimpreso, numseriepreimpresa from caja) t2 on t1.minfisico = t2.numfoliopreimpreso and t1.serie = t2.numseriepreimpresa ");
		sql.append("inner join (select numfoliosistema, numfoliopreimpreso, numseriepreimpresa from caja) t3 on t1.maxfisico = t3.numfoliopreimpreso and t1.serie = t3.numseriepreimpresa ");
		sql.append("inner join det_abasto_boleto dab on t1.minfisico between dab.numfolioinicial and dab.numfoliofinal and dab.numseriepreimpresa = t1.serie  ");
		sql.append("inner join abasto_boleto abb on abb.abastoboleto_id = dab.abastoboleto_id ");
		if (tipoventaId != null) {
			sql.append("where abb.articulo_id = " + tipoventaId);
		}
		sql.append(" order by t1.caixa, t2.numfoliosistema, t3.numfoliosistema ");
		return sql.toString();
	}

	@Override
	protected void processaParametros() throws Exception {
		Integer puntoventaId = (Integer) getParametros().get("PUNTOVENTA_ID");
		Integer tipoventaId = (Integer) getParametros().get("TIPOVENTA_ID");
		Integer empresaId = (Integer) getParametros().get("EMPRESA_ID");
		Date fecInicio = (Date) getParametros().get("DATA_INICIAL");
		Date fecFinal = (Date) getParametros().get("DATA_FINAL");
		fecInicio = DateUtil.inicioFecha(fecInicio);
		fecFinal = DateUtil.fimFecha(fecFinal);

		String sql = getSql(puntoventaId, tipoventaId);

		PreparedStatement pstmt = getConexao().prepareStatement(sql);
		pstmt.setInt(1, empresaId);
		pstmt.setTimestamp(2, new java.sql.Timestamp(fecInicio.getTime()));
		pstmt.setTimestamp(3, new java.sql.Timestamp(fecFinal.getTime()));

		List<ConferenciaFormularioFisico> formularios = new ArrayList<ConferenciaFormularioFisico>();

		ResultSet rs = pstmt.executeQuery();
		while (rs.next()) {

			ConferenciaFormularioFisico formulario = new ConferenciaFormularioFisico();
			formulario.setAbertura(rs.getDate(6));
			formulario.setBilheteFisicoAbertura(rs.getInt(8));
			formulario.setBilheteFisicoFechamento(rs.getInt(9));
			formulario.setDif1(rs.getInt(10));
			formulario.setBilheteLogicoAbertura(rs.getInt(12));
			formulario.setBilheteLogicoFechamento(rs.getInt(13));
			formulario.setDif2(rs.getInt(10));
			formulario.setCaixa(rs.getString(2));
			formulario.setFechamento(rs.getDate(7));
			formulario.setSerie(rs.getString(3));
			formulario.setTerminal(rs.getInt(1));
			formulario.setTurno(rs.getInt(5));
			formulario.setUsuario(rs.getString(4));
			formulario.setTotal(rs.getInt(11));
			formularios.add(formulario);
		}

		setLsDadosRelatorio(formularios);
	}
}
