package com.rjconsultores.ventaboletos.web.utilerias.servlet;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.core.io.ClassPathResource;

import com.rjconsultores.ventaboletos.entidad.Constante;
import com.rjconsultores.ventaboletos.service.ConstanteService;
import com.rjconsultores.ventaboletos.web.utilerias.spring.AppContext;

/**
 * Servlet implementation class ImageServlet
 */
public class ImageServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
	private static final String PATH_IMAGENES = "/com/rjconsultores/ventaboletos/web/cliente/imagenes/";
	private static final String IMAGENE_EMPRESA = "empresa.png";
	private static final String IMAGENE_EMPRESA_GENERICA = "empresa_generica.png";
	private static Boolean IMAGENE_EMPRESA_GENERICA_CHECKED = null;

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse resp) throws ServletException, IOException {
		// Set content type
		resp.setContentType("image/png");

		String imagene = request.getParameter("imagene");
		if (imagene != null) {
			// Open the file and output streams
			imagene = getNombreImagene(imagene);
			
			InputStream in = getImagene(imagene);
			OutputStream out = resp.getOutputStream();

			// Copy the contents of the file to the output stream
			byte[] buf = new byte[1024];
			int count = 0;
			while ((count = in.read(buf)) >= 0) {
				out.write(buf, 0, count);
			}
			in.close();
			out.close();
		}
	}

	/**
	 * Muda a imagem para uma imagem generica da empresa.
	 * 
	 * Utilizado para carregar a imagem padro ao invs da do custom
	 * 
	 * @param imagene
	 * @return
	 */
	private String getNombreImagene(String imagene){
		if (!imagene.equalsIgnoreCase(IMAGENE_EMPRESA)){
			return imagene;
		}
		
		boolean imageneGenericaEmpresaActivo =  false;
		
		if (IMAGENE_EMPRESA_GENERICA_CHECKED == null){
			
			IMAGENE_EMPRESA_GENERICA_CHECKED = false;
			
			ConstanteService constanteService = getConstanteService();
			Constante constante = constanteService.buscarPorNomeConstante("IMAGENE_EMPRESA_GENERICA");
			if ( (constante != null) && StringUtils.isNotBlank(constante.getValorconstante())){
				imageneGenericaEmpresaActivo= Boolean.parseBoolean(constante.getValorconstante());
				
				IMAGENE_EMPRESA_GENERICA_CHECKED = imageneGenericaEmpresaActivo;
			}
			
		}
		
		
		if ( (IMAGENE_EMPRESA_GENERICA_CHECKED != null) && (IMAGENE_EMPRESA_GENERICA_CHECKED)  ){
			imagene = IMAGENE_EMPRESA_GENERICA;
		}
		
		return imagene;
		
		
	}
	private InputStream getImagene(String imagene) {
		InputStream inputStream = null;
		try {
			inputStream = new ClassPathResource(PATH_IMAGENES + imagene).getInputStream();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return inputStream;
	}

	
	private ConstanteService getConstanteService() {
		ApplicationContext appContext = AppContext.getApplicationContext();
		BeanFactory factory = (BeanFactory) appContext;
		ConstanteService cs = (ConstanteService) factory.getBean("constanteService");
		return cs;
	}
}
